/* ------------------------------------------------------------------------------------------------ */
/* FILE :		Demo_5_LPC1227STK_ATOLLIC.c															*/
/*																									*/
/* CONTENTS :																						*/
/*				Demo for the Olimex LPC1227-STK evaluation board									*/
/*				Operation of the 3 buttons shown on the serial port									*/
/*				Serial port settings:																*/
/*							Baud Rate: 115200														*/
/*							Data bits:      8														*/
/*							Stop bits:      1														*/
/*							Parity:      none														*/
/*							Flow Ctrl:   none														*/
/*							Emulation: VT-100														*/
/*																									*/
/*																									*/
/* Copyright (c) 2012-2014, Code-Time Technologies Inc. All rights reserved.						*/
/*																									*/
/* Code-Time Technologies retains all right, title, and interest in and to this work				*/
/*																									*/
/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS							*/
/* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF										*/
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL							*/
/* THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR								*/
/* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,							*/
/* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR							*/
/* OTHER DEALINGS IN THE SOFTWARE.																	*/
/*																									*/
/*																									*/
/*	$Revision: 1.3 $																				*/
/*	$Date: 2012/06/18 14:35:07 $																	*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

#if OS_DEMO != 5
	#error "Wrong OS_DEMO value" 
#endif

#include <stdio.h>
#include <stdarg.h>
#include <ctype.h>
#include <Abassi.h>

/* ------------------------------------------------ */

#include "lpc12xx_libcfg_default.h"
#include "lpc12xx_gpio.h"
#include "lpc12xx_iocon.h"
#include "lpc12xx_sysctrl.h"
#include "lpc12xx_uart.h"

#define BAUDRATE	  115200						/* Serial port baud rate						*/
#define SYS_FREQ	24000000						/* Processor clock frequency					*/

#define UART_PORT 0
#define TEST_UART      LPC_UART0
#define TEST_UART_RXD  IOCON_UART_RXD0_LOC0
#define TEST_UART_TXD  IOCON_UART_TXD0_LOC0

IOCON_PIO_CFG_Type PIO_mode;

#define TEST_UART      LPC_UART0
#define TEST_UART_RXD  IOCON_UART_RXD0_LOC0
#define TEST_UART_TXD  IOCON_UART_TXD0_LOC0

#define puts(xxx) 		UART_Send(LPC_UART0, (unsigned char *)xxx, strlen(xxx), -1);				\
                  		UART_Send(LPC_UART0, (unsigned char *)"\r\n", 2, -1);
#define WriteStr(xxx)	UART_Send(LPC_UART0, (unsigned char *)xxx, strlen(xxx), -1);
#ifdef getchar
  #undef getchar
#endif
#ifdef putchar
  #undef putchar
#endif
void UARTInit (unsigned int BaudRate);

/* ------------------------------------------------ */

MBX_t *MbxRight;									/* Mailbox used by TaskRight					*/
SEM_t *SemCenter;									/* Semaphore for TaskCenter						*/

TSK_t *TaskCenter;									/* Task handling the joystick center activated	*/
TSK_t *TaskLeft;									/* Task handling the Left button				*/
TSK_t *TaskRight;									/* Task handling the Right button				*/
TSK_t *TaskTick;									/* Task operated one every RTOS tick			*/

int Debounce[3];									/* Counters to debolunce the buttons			*/

/* ------------------------------------------------ */

void FctCenter(void);								/* Functon attached to TaskCenter				*/
void FctLeft(void);									/* Functon attached to TaskLeft					*/
void FctRight(void);								/* Functon attached to TaskRight				*/
void FctTick(void);									/* Functon attached to TaskTick					*/
void Buttons(void);									/* Buttons interrupt handler					*/

/* ------------------------------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------------------------------ */

int main(void)
{
int  Prio = 0;										/* SysTick interrupt priority					*/
int *TimerBase;										/* Systick Timer register base adddress			*/


/* ------------------------------------------------ */
/* 	Processor clock set-up							*/

	SystemInit();
	OSdint();

/* ------------------------------------------------ */
/* 	Start the RTOS									*/

	OSstart();

/* ------------------------------------------------ */
/* UART set-up										*/

	UARTInit(BAUDRATE);

	setvbuf(stdin,  NULL, _IONBF, 0);
	setvbuf(stdout, NULL, _IONBF, 0);

/* ------------------------------------------------ */
/* SysTick Set-up									*/

	TimerBase = (int *)0xE000E000;
	((volatile int *)0xE000Ed20)[0] = (((volatile int *)0xE000Ed20)[0] & 0x1FFFFFFF)
	                                |   ((Prio & 7) << 29);
	OSisrInstall(-1, TIMtick);						/* Install the OS timer tick function			*/
	TimerBase[5] = (((SYS_FREQ/100000) * OS_TIMER_US)/10)-1;
	TimerBase[4] = 0x7;								/* internal clock / int & enable				*/

/* ------------------------------------------------ */
/* GPIO (Switches) set-up							*/

    SYS_ConfigAHBCLK(SYS_AHBCLKCTRL_GPIO2, ENABLE);
    NVIC_EnableIRQ(EINT2_IRQn);

    IOCON_StructInit(&PIO_mode);

	PIO_mode.type = IOCON_PIO_2_12;					/* USER1 switch in input						*/
	IOCON_SetFunc(&PIO_mode);
	GPIO_SetDir(LPC_GPIO2, 12, 0);
	GPIO_SetLowLevel(LPC_GPIO2, 12, 1);
	GPIO_IntSetType( LPC_GPIO2, 12, GPIO_INTERRUPT_FALLING);
	GPIO_IntSetMask( LPC_GPIO2, 12, 1 );

	PIO_mode.type = IOCON_PIO_2_11;					/* USER2 switch in input						*/
	IOCON_SetFunc(&PIO_mode);
	GPIO_SetDir(LPC_GPIO2, 11, 0);
	GPIO_SetLowLevel(LPC_GPIO2, 11, 1);
	GPIO_IntSetType( LPC_GPIO2, 11, GPIO_INTERRUPT_FALLING);
	GPIO_IntSetMask( LPC_GPIO2, 11, 1 );

	PIO_mode.type = IOCON_PIO_2_10;					/* USER3 switch in input						*/
	IOCON_SetFunc(&PIO_mode);
	GPIO_SetDir(LPC_GPIO2, 10, 0);
	GPIO_SetLowLevel(LPC_GPIO2, 10, 1);
	GPIO_IntSetType( LPC_GPIO2, 10, GPIO_INTERRUPT_FALLING);
	GPIO_IntSetMask( LPC_GPIO2, 10, 1 );

	OSisrInstall(27, Buttons);

/* ------------------------------------------------ */
/* Application set-up								*/

	TaskCenter    = TSKcreate("Center", 1, TC_TSK_STACK, &FctCenter, 1);
	TaskLeft      = TSKcreate("Left  ", 1, TC_TSK_STACK, &FctLeft,   1);
	TaskRight     = TSKcreate("Right ", 1, TC_TSK_STACK, &FctRight,  1);
	TaskTick      = TSKcreate("Tick  ", 1, TC_TSK_STACK, &FctTick,   1);

	MbxRight    = MBXopen("Right", 3);				/* Create the mailbox used by FctRight() 		*/

	SemCenter   = SEMopen("Center");				/* Create the semaphore used by FctCenter()		*/

	Debounce[0] = 0;								/* Set-up the debounce counter for each pin		*/
	Debounce[1] = 0;
	Debounce[2] = 0;

/* ------------------------------------------------ */

	puts("\n\n");									/* Welcome banner								*/
	puts("\n           Abassi Demo 5");
	puts("\n          Olimex LPC-1227\n");
	puts("\n         Use the 3 Buttons\n\n");


/* ------------------------------------------------ */
/* Interrupts ON, the application runs				*/

	OSeint(1);

	for (;;) {
		TSKselfSusp();								/* And Adam & Eve is done, it self suspends		*/
	}
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* Idle Task																						*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

void IdleTask(void)
{

	for(;;);										/* Idle is a do nothing task					*/

}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* Task functions																					*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

void FctCenter(void)
{
	for (;;) {
		SEMwaitBin(SemCenter, -1);					/* Get blocked waiting for the joystick center	*/
													/* to be activated								*/
		WriteStr("    Center\r");
	}
}


/* ------------------------------------------------------------------------------------------------ */

void FctLeft(void)
{
int events;

	for (;;) {
		EVTwait(0xFFFF, 0xFFFF, -1);				/* Wait for the event flags to be set by ISR	*/
		events = EVTget();
		if ((0x01 & events) != 0) {					/* Flag bit #0 is for right position			*/
			WriteStr("    Left  \r");
		}
	}
}

/* ------------------------------------------------------------------------------------------------ */

void FctRight(void)
{
intptr_t Msg;

	for (;;) {
		MBXget(MbxRight, &Msg, -1);


		if ((int)Msg == 1) {						/* Message #1 is for right button				*/
			WriteStr("    Right \r");
		}
	}
}

/* ------------------------------------------------------------------------------------------------ */

void FctTick(void)
{
int   ii;
const char Running[] = "|/-\\";

	ii = 0;
	for (;;) {
		EVTwait(0xFFFF, 0xFFFF, -1);				/* Wait for any event flag to be set by the		*/
													/* RTOS tierm callback							*/
		putchar(Running[ii++]);
		putchar('\r');
		ii &= 3;
	}
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* Joystick interrupt handler function																*/
/* 																									*/
/* ------------------------------------------------------------------------------------------------ */

void Buttons(void)
{

	if ((GPIO_IntGetMaskStatus(LPC_GPIO2, 12))		/* USER 1 button								*/
	&&  (Debounce[0] == 0)) {						/* Right button									*/
		MBXput(MbxRight, 1, 0);
		Debounce[0] = OS_TICK_PER_SEC/50;
        GPIO_IntClear(LPC_GPIO2, 12, 1 );
	}

	if ((GPIO_IntGetMaskStatus(LPC_GPIO2, 11))		/* USER 2 button								*/
	&&  (Debounce[1] == 0)) {						/* Center button								*/
		SEMpost(SemCenter);
		Debounce[1] = OS_TICK_PER_SEC/50;
        GPIO_IntClear(LPC_GPIO2, 11, 1 );
	}
	if ((GPIO_IntGetMaskStatus(LPC_GPIO2, 10))		/* USER 3 button								*/
	&&  (Debounce[2] == 0)) {						/* Left button									*/
		EVTset(TaskLeft, 0x01);
		Debounce[2] = OS_TICK_PER_SEC/50;
        GPIO_IntClear(LPC_GPIO2, 10, 1 );
	}

	return;
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* Timer call back function																			*/
/* 																									*/
/* ------------------------------------------------------------------------------------------------ */

void TIMcallBack(void)
{
static int CBcnt=0;


	if (++CBcnt > (OS_TICK_PER_SEC/10)) {			/* Trigger FctTick()once every 0.1 s			*/
		CBcnt=0;
		EVTset(TaskTick, 1);
	}

	if (GPIO_GetPinValue(LPC_GPIO2, 12)				/* Debouncing									*/
	&&  (Debounce[0] != 0)) {
		Debounce[0]--;
	}

	if (GPIO_GetPinValue(LPC_GPIO2, 11)				/* Debouncing									*/
	&&  (Debounce[1] != 0)) {
		Debounce[1]--;
	}
	if (GPIO_GetPinValue(LPC_GPIO2, 10)				/* Debouncing									*/
	&&  (Debounce[2] != 0)) {
		Debounce[2]--;
	}

	return;
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* UART functions needed by the Peripheral library to access the UART								*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

void UARTInit(unsigned int BaudRate)
{
UART_CFG_Type      UARTConfigStruct;
IOCON_PIO_CFG_Type PIO_mode;


	IOCON_StructInit(&PIO_mode);
	PIO_mode.type = TEST_UART_RXD;
	IOCON_SetFunc(&PIO_mode);
	PIO_mode.type = TEST_UART_TXD;
	IOCON_SetFunc(&PIO_mode);
  
	SYS_ResetPeripheral(SYS_PRESETCTRL_UART0_RST,DISABLE);
	SYS_ConfigAHBCLK(SYS_AHBCLKCTRL_UART0, ENABLE); 
	SYS_SetUART0ClockDiv(1);

	UART_Init(TEST_UART);

	UART_GetConfig(TEST_UART, &UARTConfigStruct);
	UARTConfigStruct.baudrate = BaudRate;
	UARTConfigStruct.databits = UART_CFG_DATABIT_8;

	UART_SetConfig(TEST_UART, &UARTConfigStruct);
	UART_ConfigTXD(TEST_UART, ENABLE);

	return;
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* UART functions needed by the Peripheral library to access the UART								*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

_ssize_t _write_r (struct _reent *rreenn, int fd, const void *vbuf, size_t len)
{
const char *buf;
int size;

	buf = vbuf;
	size = len;
	for ( ; size > 0 ; size--) {
		if (*buf == '\n') {
			UART_Send(LPC_UART0, (unsigned char *)"\r", 1, -1);
		}
		UART_Send(LPC_UART0, (unsigned char *)buf++, 1, -1);
	}

	return(len);
}

/* ------------------------------------------------------------------------------------------------ */

_ssize_t  _read_r(struct _reent *rreenn, int fd, void *vbuf, size_t size)
{
int RetVal;


	RetVal = UART_Receive(LPC_UART0, (unsigned char *)vbuf, 1, -1);

	return(RetVal);
}

/* ------------------------------------------------------------------------------------------------ */

int __putchar(int ccc)
{

	if (ccc == '\n') {
		UART_Send(LPC_UART0, (unsigned char *)'\r', 1, -1);
	}
	UART_Send(LPC_UART0, (unsigned char *)ccc, 1, -1);

	return(ccc);
}


/* ------------------------------------------------------------------------------------------------ */

int GetKey(void)
{
unsigned char ccc;
int           RetVal;


	RetVal = UART_Receive(LPC_UART0, &ccc, 1, 1);
	if (RetVal != 0) {
		RetVal = (int)ccc;
	}
	return(RetVal);
}

/* EOF */
